

class Town
{
	id = 0;
	rate = 0;
	neighbors = null;

	constructor(town_id)
	{
		this.id = town_id;
		this.rate = 1;
		this.neighbors = [];

		local all_towns = GoalTownList();

		foreach(t, _ in all_towns)
		{
			/* Skip town if it is the same as this.id */
			if(t == this.id)
				continue;

			/* Add towns with distance less than 100 tiles to neighbor list */
			local distance = GoalMap.DistanceManhattan(GoalTown.GetLocation(town_id), GoalTown.GetLocation(t));
			if(distance < 100)
			{
				neighbors.append( { id = t, dist = distance } );
			}
		}

		/* Sort by distance */
		neighbors.sort(TownDistSorter);

		/* Remove the most distant towns, so that only three neighbors remain */
		while(neighbors.len() > 3)
		{
			neighbors.pop();
		}
	}

	function CalculateGrowTime();
	function SetGoal();
}

function TownDistSorter(a, b) { 
	return a.dist == b.dist? 1 : (a.dist > b.dist ? 1 : -1); 
}

function Town::CalculateGrowTime()
{
	local rating = GoalTown.TOWN_RATING_NONE;
	for (local companyID = GoalCompany.COMPANY_FIRST; companyID <= GoalCompany.COMPANY_LAST; companyID++) {
		if (GoalCompany.ResolveCompanyID(companyID) != GoalCompany.COMPANY_INVALID) {
			local r = GoalTown.GetRating(this.id, companyID);
			if (r > rating) rating = r;
		}
	}

	local rate = 420;
	switch (rating) {
		case GoalTown.TOWN_RATING_NONE:        rate = 420; break;
		case GoalTown.TOWN_RATING_APPALLING:   rate = 360; break;
		case GoalTown.TOWN_RATING_VERY_POOR:   rate = 300; break;
		case GoalTown.TOWN_RATING_POOR:        rate = 260; break;
		case GoalTown.TOWN_RATING_MEDIOCRE:    rate = 220; break;
		case GoalTown.TOWN_RATING_GOOD:        rate = 190; break;
		case GoalTown.TOWN_RATING_VERY_GOOD:   rate = 160; break;
		case GoalTown.TOWN_RATING_EXCELLENT:   rate = 130; break;
		case GoalTown.TOWN_RATING_OUTSTANDING: rate = 100; break;
	}

	local town_growth_rate = GoalGameSettings.GetValue("economy.town_growth_rate") - 1;
	if (town_growth_rate < 1) town_growth_rate = 1;

	rate = rate >> town_growth_rate;
	rate = rate / (GoalTown.GetPopulation(this.id) / 250 + 1);

	if (rate < 1) rate = 1;
	GoalTown.SetGrowthRate(this.id, rate);
	return rate;
}

function Town::SetGoal()
{
	local population = GoalTown.GetPopulation(this.id);

	local n_factor = this.GetNeighborhoodFactor(population);

	switch (GoalGame.GetLandscape()) {
		case GoalGame.LT_TEMPERATE:
		case GoalGame.LT_TOYLAND:
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_PASSENGERS, max(((population / 10) - 5) * n_factor / 100, 1));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_MAIL,       max(((population / 50) - 10) * n_factor / 100, 0));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_GOODS,      max(((population / 100) - 10) * n_factor / 100, 0));
			break;

		case GoalGame.LT_ARCTIC:
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_PASSENGERS, max(((population / 10) - 5) * n_factor / 100, 1));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_FOOD,       max(((population / 20) - 5) * n_factor / 100, 0));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_MAIL,       max(((population / 50) - 10) * n_factor / 100, 0));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_GOODS,      max(((population / 100) - 20) * n_factor / 100, 0));
			break;

		case GoalGame.LT_TROPIC:
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_PASSENGERS, max(((population / 10) - 5) * n_factor / 100, 1));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_FOOD,       max(((population / 20) - 5) * n_factor / 100, 0));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_WATER,      max(((population / 50) - 2) * n_factor / 100, 0));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_MAIL,       max(((population / 50) - 10) * n_factor / 100, 0));
			GoalTown.SetCargoGoal(this.id, GoalCargo.TE_GOODS,      max(((population / 100) - 20) * n_factor / 100, 0));
			break;
	}
}

function Town::GetNeighborhoodFactor(self_population)
{
	local factor = 100;

	foreach(n in this.neighbors)
	{
		local diff = GoalTown.GetPopulation(n.id) - self_population; 
		factor -= diff * 3;
	}

	factor = max(factor, 70);
	factor = min(factor, 800);

	//GoalLog.Info(GoalTown.GetName(this.id) + " has factor " + factor);

	return factor;
}
